@@ -1,11 +1,12 @@
+import contextlib
 import gc
+import json
 import logging
+import mmap
 import os
 import shutil
-import subprocess
 import sys
 from pathlib import Path
-from sys import version_info
 
 import numpy as np
 import pandas as pd
@@ -55,9 +56,67 @@ def open_many(paths):
     return concat_vaex
 
 
-def read_data(path, prefix='', suffix=''):
+def read_file(path):
+    if path.endswith('.csv'):
+        read = pd.read_csv
+    elif path.endswith('.parquet'):
+        read = pd.read_parquet
+    elif path.endswith('.feather'):
+        read = pd.read_feather
+    elif path.endswith(('.hdf', '.hdf5', '.h5')):
+        read = pd.read_hdf
+    elif path.endswith(('.pkl', '.pickle', '.p')):
+        read = pd.read_pickle
+    elif path.endswith('.json'):
+        try:
+            with open(path, 'r') as f:
+                path = json.load(f)
+            read = pd.read_json
+        except Exception as e:
+            return None
+    elif path.endswith('txt'):
+        path = Path(path).read_text()
+        read = pd.read_json
+    else:
+        return None
+    with contextlib.suppress():
+        return read(path)
+    return None
+
+
+def read_pandas_files(files):
+    if len(files) == 0:
+        logger.error(f"found no data files")
+        return None
+    bad_files, valid_files = [], []
+
+    for path in files:
+        df = read_file(path)
+        if df is not None:
+            valid_files.append(df)
+        else:
+            logger.error(f"file {path} is broken")
+            bad_files.append(path)
+
+    logger.info(f"found {len(bad_files)} malformed files")
+    logger.info(f"opened {len(valid_files)} files")
+    return pd.concat(valid_files, ignore_index=True)
+
+
+def read_sklearn_data(path, prefix='', suffix='', shuffle=True):
+    files = [str(path) for path in Path(path).rglob(f"{prefix}*{suffix}")]
+    logger.info(f"relevant files: {files}")
+    logger.info(f"found {len(files)} files")
+    df = read_pandas_files(files)
+    if df is not None and shuffle:
+        df = df.sample(frac=1).reset_index(drop=True)
+    if df is not None:
+        logger.info(f"data shape {df.shape}")
+    return df
+
+
+def read_vaex_data(path, prefix='', suffix='', shuffle=True):
     import vaex
-    prefix = prefix or ''
     logger.info(f"read data from {path} and prefix {prefix} and suffix {suffix}")
     if os.path.isdir(path):
         files = [str(path) for path in Path(path).rglob(f"{prefix}*{suffix}")]
@@ -67,21 +126,41 @@ def read_data(path, prefix='', suffix=''):
             logger.error(f"found no data files")
             return None
 
-        bad_files = []
-        for file in files:
-            try:
-                if vaex.open(file).head(2):
-                    pass
-            except:
-                logger.error(f"file {file} is broken")
-                bad_files.append(file)
+        vaex_files_paths, pandas_files, malformed_files = [], [], []
 
-        paths = list(set([file for file in files if file not in bad_files]))
-        logger.info(f"found {len(bad_files)} malformed files")
-        logger.info(f"open {len(paths)} files")
-        df = vaex.open_many(paths)
+        for path in files:
+            if not path.endswith(('arrow', 'csv', 'hdf5', 'parquet', 'feather')):
+                df = read_file(path)
+                if df is not None:
+                    pandas_files.append(df)
+                continue
+            try:
+                vaex.open(path).head(2)
+                vaex_files_paths.append(path)
+            except Exception as e:
+                try:
+                    pandas_files.append(read_file(path))
+                except:
+                    malformed_files.append(path)
+
+        logger.info(
+            f"found {len(malformed_files)} malformed files - {len(vaex_files_paths)} vaex files - {len(pandas_files)} pandas files")
+        df = None
+        if len(vaex_files_paths) > 0:
+            df = vaex.open_many(vaex_files_paths)
+        if pandas_files:
+            from_pandas_df = vaex.from_pandas(pd.concat(pandas_files, ignore_index=True))
+            if df is None:
+                df = from_pandas_df
+            else:
+                df = vaex.concat([df, from_pandas_df])
     else:
-        df = vaex.open(path, shuffle=True)
+        try:
+            df = vaex.open(path, shuffle=shuffle)
+        except:
+            df = vaex.from_pandas(read_file(path))
+            if shuffle:
+                df = df.shuffle()
     logger.info(f"data shape {df.shape}")
     return df
 
@@ -139,18 +218,25 @@ def validate_path(path):
     return ret
 
 
-def read_bytes(path):
+def remove_signeture(s):
+    return s[len(CONSTANTS.BYTES_SIGNETURE):]
+
+
+def add_signeture(s):
+    return CONSTANTS.BYTES_SIGNETURE + s
+
+
+def read_meta_bytes(path):
     open_fs = get_open(path)
+    with open_fs(path, 'r+') as f:
+        with mmap.mmap(f.fileno(), 0) as mf:
+            offset = mf.find(CONSTANTS.BYTE_DELIMITER)
+            if offset == -1:
+                raise Exception(f"{CONSTANTS.BYTE_DELIMITER} not found")
+        f.seek(offset)
     with open_fs(path, 'rb') as f:
-        ret = f.read()
-    return ret
-
-
-def write_bytes(path, bytes_to_write):
-    open_fs = get_open(path)
-    with open_fs(path, "wb") as outfile:
-        outfile.write(bytes_to_write)
-    return path
+        meta_bytes = f.read(offset)
+    return meta_bytes
 
 
 def get_open(path):
@@ -179,38 +265,13 @@ def unpickle(b):
     return ret
 
 
-def get_requirements(venv_type=None, requirements=None, clean=True):
-    """Run pip freeze  for venv and conda env export for conda
-    @return requirements
-    """
-    if requirements is not None:
-        return '\n'.join(requirements)
-    if venv_type is None:
-        venv_type = get_env_type()
-    if venv_type == 'conda':
-        command = ["conda env export | cut -f 1 -d '=' "]
-        env = subprocess.check_output(command, shell=True).decode()
-        if clean:
-            env = env.replace('\n  - appnope', '')
-        splited = env.split('\n')
-        splited[0] = 'name: conda_env'
-        splited[-2] = 'prefix: conda_env'
-        return '\n'.join(splited)
-    ret = subprocess.check_output([sys.executable, '-m', 'pip',
-                                   'freeze']).decode()
-    if clean:
-        import re
-        ret = re.sub("appnope==(.[\d \.]*)\\n", '', ret)
-    return ret
-
-
 def get_python_version():
     """
     @return: current python version
     """
-    return "{major}.{minor}.{micro}".format(major=version_info.major,
-                                            minor=version_info.minor,
-                                            micro=version_info.micro)
+    return "{major}.{minor}.{micro}".format(major=sys.version_info.major,
+                                            minor=sys.version_info.minor,
+                                            micro=sys.version_info.micro)
 
 
 def get_env_type():
@@ -222,18 +283,3 @@ def get_env_type():
     elif os.getenv('VIRTUAL_ENV'):
         return 'venv'
     return None
-
-
-def get_conda_env(clean=True):
-    """run conda env export | cut -f 1 -d '='  and clean problematic packages (for docker) like 'appnope'"""
-    env = None
-    if get_env_type() == 'conda':
-        command = ["conda env export | cut -f 1 -d '=' "]
-        env = subprocess.check_output(command, shell=True).decode()
-        if clean:
-            env = env.replace('\n  - appnope', '')
-        splited = env.split('\n')
-        splited[0] = 'name: conda_env'
-        splited[-2] = 'prefix: conda_env'
-        env = '\n'.join(splited)
-    return env
